//-*-c++-*-
#ifndef INCLUDED_ToggleHeadLightBehavior_h_
#define INCLUDED_ToggleHeadLightBehavior_h_

#include "Shared/SharedObject.h"
#include "Motion/PostureMC.h"
#include "Shared/ERS220Info.h"

//! opens or closes the head light on an ERS-220
class ToggleHeadLightBehavior : public BehaviorBase {
public:
	//! constructor
	ToggleHeadLightBehavior() : BehaviorBase(), light_id(MotionManager::invalid_MC_ID) {}

	//! opens the head light
	virtual void DoStart() {
		BehaviorBase::DoStart();
		if(state->robotDesign & WorldState::ERS220Mask) {
			SharedObject<PostureMC> pose;
			pose->setOutputCmd(ERS220Info::RetractableHeadLEDOffset,true);
			light_id=motman->addMotion(pose,false);
		}
	}

	//! resets the head light
	virtual void DoStop() {
		motman->removeMotion(light_id);
	}

	static std::string getClassDescription() { return "Opens or closes the head light on an ERS-220"; }

	std::string getName() const { return "ToggleHeadLightBehavior"; }

protected:
	MotionManager::MC_ID light_id;
};

/*! @file
 * @brief Defines ToggleHeadLightBehavior, which will open or close the head light on an ERS-220
 * @author ejt (Creator)
 *
 * $Author: ejt $
 * $Name: tekkotsu-1_4 $
 * $Revision: 1.1 $
 * $State: Exp $
 * $Date: 2003/06/28 18:03:57 $
 */

#endif
