#include "StartupBehavior.h"

#include "Behaviors/Controller.h"
#include "Shared/SharedObject.h"

#include "Behaviors/Controls/BatteryCheckControl.h"
#include "Behaviors/Controls/BehaviorActivatorControl.h"
#include "Behaviors/Controls/BehaviorSwitchControl.h"
#include "Behaviors/Controls/ControlBase.h"
#include "Behaviors/Controls/DumpFileControl.h"
#include "Behaviors/Controls/FileBrowserControl.h"
#include "Behaviors/Controls/FreeMemReportControl.h"
#include "Behaviors/Controls/HelpControl.h"
#include "Behaviors/Controls/LoadPostureControl.h"
#include "Behaviors/Controls/LoadWalkControl.h"
#include "Behaviors/Controls/MCValueEditControl.h"
#include "Behaviors/Controls/PlaySoundControl.h"
#include "Behaviors/Controls/ProfilerCheckControl.h"
#include "Behaviors/Controls/RebootControl.h"
#include "Behaviors/Controls/RunSequenceControl.h"
#include "Behaviors/Controls/SavePostureControl.h"
#include "Behaviors/Controls/SaveWalkControl.h"
#include "Behaviors/Controls/ShutdownControl.h"
#include "Behaviors/Controls/ValueEditControl.h"
#include "Behaviors/Controls/ValueSetControl.h"

#include "Behaviors/Controls/EventLogger.h"

#include "Behaviors/Demos/AutoGetupBehavior.h"
#include "Behaviors/Demos/BatteryMonitorBehavior.h"
#include "Behaviors/Demos/ChaseBallBehavior.h"
#include "Behaviors/Demos/SimpleChaseBallBehavior.h"
#include "Behaviors/Demos/StareAtBallBehavior.h"
#include "Behaviors/Demos/FollowHeadBehavior.h"
#include "Behaviors/Demos/HeadLevelBehavior.h"
#include "Behaviors/Demos/EvtRptBehavior.h"
#include "Behaviors/Demos/WalkToTargetMachine.h"
#include "Behaviors/Demos/BanditMachine.h"
#include "Behaviors/Demos/WorldModel2Behavior.h"
#include "Behaviors/Demos/DumbWM2Behavior.h"
#include "Behaviors/Demos/SoundTestBehavior.h"
#include "Behaviors/Demos/ToggleHeadLightBehavior.h"
#include "Behaviors/Demos/WalkControllerBehavior.h"
#include "Behaviors/Demos/HeadPointControllerBehavior.h"
#include "Behaviors/Demos/Aibo3DControllerBehavior.h"
#include "Behaviors/Demos/Aibo3DMonitorBehavior.h"
#include "Behaviors/Demos/EStopControllerBehavior.h"

#include "Motion/MotionCommand.h"
#include "Motion/PostureMC.h"
#include "Motion/EmergencyStopMC.h"
#include "Motion/PIDMC.h"
#include "Motion/MotionSequenceMC.h"
#include "Motion/MMAccessor.h"
#include "Shared/ERS210Info.h"

#include "SoundPlay/SoundManager.h"

StartupBehavior gStartup; //!< used to initialize the global ::startupBehavior, used by MMCombo
BehaviorBase& startupBehavior=gStartup; //!< used by MMCombo as the init behavior

StartupBehavior::StartupBehavior()
	: BehaviorBase(), spawned(),
		stop_id(MotionManager::invalid_MC_ID),
		pid_id(MotionManager::invalid_MC_ID)
{
	AddReference(); // this is a global, so there's a global reference
}

StartupBehavior::~StartupBehavior() {cout << "Goodbye" << endl;}

void StartupBehavior::DoStart() {
	BehaviorBase::DoStart();
	
	//This will "fade" in the PIDs so the joints don't jerk to full power, also looks cooler
	pid_id=motman->addMotion(SharedObject<PIDMC>(0),MotionManager::kEmergencyPriority+1,false);
	//also, pause before we start fading in, PIDs take effect right away, before the emergencystop is picked up
	erouter->addTimer(this,0,4*FrameTime*NumFrames,true);

	//This is the default emergency stop
	const SharedObject<EmergencyStopMC> stop;
	stop->LoadFile("/ms/data/motion/liedown.pos"); //This *should* be replaced by the current position, but just in case, better than setting everything to 0's
	stop->setStopped(true,false); //if you want to start off paused
	//sndman->StopPlay(); //so it doesn't play the halt sound the first time (but the false in previous line does that now)
	stop_id=motman->addMotion(stop,MotionManager::kEmergencyPriority);
	
	//This displays the current battery conditions
	BatteryCheckControl batchk;
	batchk.activate(MotionManager::invalid_MC_ID,NULL);
	//	const SharedObject<LedMC> led; //! @todo LedMC's don't support autopruning yet, it should for uses like this, or could the one in Controller
	//	batchk.activate(motman->addMotion(led,true));
	batchk.deactivate();

	//This is for the menu system
	Controller * controller=new Controller;
	controller->DoStart();
	controller->setEStopID(stop_id);
	controller->setRoot(SetupMenus());
	wireless->setReceiver(sout, Controller::console_callback);
	spawned.push_back(controller);
	
	sndman->PlayFile("roar.wav");
	
	//This will close the mouth so it doesn't look stupid
	//Now done by setting the emergency stop directly in processEvent, but left as demo code:
	/*	const SharedObject<PostureMC> closemouth;
			closemouth->setJointCmd(MouthOffset,outputRanges[MouthOffset][MaxRange],1);
			motman->addMotion(closemouth,MotionCommand::kEmergencyPriority+2,true);
	*/
}

void StartupBehavior::DoStop() {
	for(std::vector<BehaviorBase*>::iterator it=spawned.begin(); it!=spawned.end(); it++)
		(*it)->DoStop();
	motman->removeMotion(stop_id);
	BehaviorBase::DoStop();
}

/*!Uses a few timer events at the beginning to fade in the PID values, and closes the mouth too*/
void StartupBehavior::processEvent(const EventBase&) {
	static unsigned int start_time=-1U;
	const unsigned int tot_time=2047; //if this is set to 2048, i sometimes get funny errors: [oid:80000034,prio:1] AGRMSDriver::SetGain() : 0x0A IS USED FOR GAIN SHIFT VALUE.
	if(start_time==-1U) { //first time
		start_time=get_time();
		MMAccessor<EmergencyStopMC>(stop_id)->takeSnapshot(); //take new snapshot with hopefully valid data
	} else {
		float power=(get_time()-start_time)/(float)tot_time;
		if(power>1)
			power=1;
		{ MMAccessor<PIDMC>(pid_id)->setAllPowerLevel(power); }
		if(state->robotDesign & WorldState::ERS210Mask)
			{ MMAccessor<EmergencyStopMC>(stop_id)->setOutputCmd(ERS210Info::MouthOffset,outputRanges[ERS210Info::MouthOffset][MaxRange]); }
	}
	if((get_time()-start_time)>=tot_time) {
		erouter->removeTimer(this);
		motman->removeMotion(pid_id);
		pid_id=MotionManager::invalid_MC_ID;
	}
}

/*class WalkToPinkBallFactory : public Factory<WalkToTargetMachine> {
	public:
	static WalkToTargetMachine* construct() { return new WalkToTargetMachine(VisionEventNS::PinkBallSID); }
	};
*/

ControlBase*
StartupBehavior::SetupMenus() {
	std::cout << "CONTROLLER-INIT..." << std::flush;
	std::stack< ControlBase* > mcsetup;
	{
		ControlBase* root=new ControlBase("Root Control");
		mcsetup.push(root);
		mcsetup.top()->pushSlot(new ControlBase("Mode Switch","Contains the \"major\" applications - mutually exclusive selection"));
		{
			mcsetup.push(mcsetup.top()->getSlots().back());
			BehaviorSwitchControlBase::BehaviorGroup * bg = new BehaviorSwitchControlBase::BehaviorGroup();
			//put behaviors here
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<FollowHeadBehavior>("FollowHeadBehavior",bg,false));
			//			mcsetup.top()->pushSlot(new BehaviorSwitchControl<CameraBehavior>("Camera",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<SoundTestBehavior>("SoundTestBehavior",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<ChaseBallBehavior>("ChaseBallBehavior",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<SimpleChaseBallBehavior>("SimpleChaseBallBehavior",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<StareAtBallBehavior>("StareAtBallBehavior",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<WalkToTargetMachine,Factory1Arg<WalkToTargetMachine,VisionEventNS::VisionSourceID_t,VisionEventNS::PinkBallSID> >("WalkToPinkBall",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<BanditMachine>("BanditMachine",bg,false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<WorldModel2Behavior>("WorldModel2Behavior",bg,false));
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new ControlBase("Background Behaviors","Background daemons and monitors"));
		{ 
			mcsetup.push(mcsetup.top()->getSlots().back());
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<AutoGetupBehavior>("AutoGetupBehavior",false))->start());
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<BatteryMonitorBehavior>("BatteryMonitorBehavior",false))->start());
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<EvtRptBehavior>("EvtRptBehavior",false));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<DumbWM2Behavior>("DumbWM2Behavior",false));
			BehaviorSwitchControlBase::BehaviorGroup * aibo3D_bg = new BehaviorSwitchControlBase::BehaviorGroup();
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<Aibo3DControllerBehavior>("Aibo3DController",aibo3D_bg,false)));
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<Aibo3DMonitorBehavior>("Aibo3DMonitor",aibo3D_bg,false)));
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<HeadLevelBehavior>("HeadLevelBehavior",false));
			if(state->robotDesign & WorldState::ERS220Mask)
				mcsetup.top()->pushSlot(new BehaviorSwitchControl<ToggleHeadLightBehavior>("ToggleHeadLightBehavior",false));
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new ControlBase("TekkotsuMon","Servers for GUIs"));
		{ 
			mcsetup.push(mcsetup.top()->getSlots().back());
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<HeadPointControllerBehavior>("Head Remote Control",false)));
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<WalkControllerBehavior>("Walk Remote Control",false)));
			BehaviorSwitchControlBase::BehaviorGroup * aibo3D_bg = new BehaviorSwitchControlBase::BehaviorGroup();
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<Aibo3DControllerBehavior>("Aibo3D Controller",aibo3D_bg,false)));
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<Aibo3DMonitorBehavior>("Aibo3D Monitor",aibo3D_bg,false)));
			EStopControllerBehavior * ESCB=new EStopControllerBehavior(stop_id);
			mcsetup.top()->pushSlot((new BehaviorSwitchControl<EStopControllerBehavior,Factory1Arg<EStopControllerBehavior,MotionManager::MC_ID,MotionManager::invalid_MC_ID> >("EStop Remote Control",ESCB))->start());
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new ControlBase("Status Reports","Displays information about the runtime environment on the console"));
		{ 
			mcsetup.push(mcsetup.top()->getSlots().back());
			mcsetup.top()->pushSlot(new BatteryCheckControl());
			mcsetup.top()->pushSlot(new ProfilerCheckControl());
			mcsetup.top()->pushSlot(new EventLogger());
			FreeMemReportControl * tmp=new FreeMemReportControl();
			tmp->DoStart();
			mcsetup.top()->pushSlot(tmp);
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new ControlBase("File Access","Access/load files on the memory stick"));
		{ 
			mcsetup.push(mcsetup.top()->getSlots().back());
			mcsetup.top()->pushSlot(new LoadPostureControl("Load Posture",stop_id));
			mcsetup.top()->pushSlot(new SavePostureControl("Save Posture"));
			mcsetup.top()->pushSlot(new RunSequenceControl<MotionSequence::SizeXLarge>("Run Motion Sequence",stop_id));
			mcsetup.top()->pushSlot(new PlaySoundControl("Play Sound"));
			mcsetup.top()->pushSlot(new DumpFileControl("Files","/ms"));
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new ControlBase("Shutdown?"));
		{ 
			mcsetup.push(mcsetup.top()->getSlots().back());
			mcsetup.top()->pushSlot(new ShutdownControl());
			mcsetup.top()->pushSlot(new RebootControl());
			mcsetup.pop();
		}
		mcsetup.top()->pushSlot(new HelpControl(root));
	}
	if(mcsetup.size()!=1)
		cout << "*** WARNING *** menu setup stack more than one or empty" << endl;
	cout << "DONE" << endl;
	return mcsetup.top();
}

			/*			XORBehavior* xorbeh=new XORBehavior(); //i make other controls which reference xorbeh below
			mcsetup.top()->pushSlot(new BehaviorSwitchControl<XORBehavior>("AI Control",xorbeh,bg));
			{ // I don't necessarily like this method of adding sub menus to control behavior parameters, but you can do it quickly. (cleaner to subclass the behavior, have it add its own)
				mcsetup.push(mcsetup.top()->getSlots().back());
				mcsetup.top()->pushSlot(new BehaviorActivatorControl("Toggle",(BehaviorSwitchControlBase*)mcsetup.top()));
				mcsetup.top()->pushSlot(new ValueSetControl<float>("Enable learning (0.035)",&xorbeh->getModel()->learn_rate,0.035));
				mcsetup.top()->pushSlot(new ValueSetControl<float>("Disable learning",&xorbeh->getModel()->learn_rate,0));
				mcsetup.top()->pushSlot(new LoadAIControl("Load AI",xorbeh->getModel()));
				mcsetup.top()->pushSlot(new SaveAIControl("Save AI",xorbeh->getModel()));
				mcsetup.pop();
			}
			*/
